# gokuemu 0.5 by gabriele@favrin.net
#
# This script emulates few annoying but funny features
# of the mIRC GOKU script, such as "Il saggio" and
# "Fortune teller" (soon).
# Thanks to seyia@irc for providing the stuff.
#
# History:
# 0.0  - Definition of the script structure.
# 0.1  - Added load of library of definitions for "Saggio".
#        Added routines to manage !saggio trigger.
#        Added replacement of %you with nick and %owner
#        with channel owner as defined in the script.
#        Added placeholders %owner (chan owner) and %you
#        (nick who did the question).
# 0.2  - Added duplicated text check: you can't do the
#        same question two times in a row...
#        Added a little antiabuse feature: questions by
#        |B people aren't answered.
#        Added placeholder %someone to get a random
#        nick from the channel.
# 0.3  - Tried to add indipendent chan settings support
#        using lists. Not the best solution as lreplace
#        gives me some problems. Will switch to arrays...
# 0.4  - Successfully switched to an array solution for
#        channel settings management... now this is a true
#        multichan script!
#        Semplified some routines by putting all the checks
#        (validchan, isonchan, matchattr) in the same "if".
#        Added multichan support for the anti dupe check too:
#        you can do the same wuestion two times in a row
#        but only on different channels.
#        Removed old and unused vars.
#        Added placeholder %number which gives a random number (up to 60 for time jokes)
#        Fixed emulation of "Saggio" layout:
#        the proper is 'nick, "header: test"', not 'header: text'.
# 0.5  - Made automatic the creation of the array for lastmsg...
#        (I've learned the use of 'foreach' ;-)
#        Removed some unused code left in get_rnd_msg
#        Removed goku_channels list: all the channel management
#        is done using arrays and there is is space for optiomization.
#        Fixed again layout: the right one is 'nick, header: "text"'.
#

#
# Preferences
#

# Set these vars for any channel the script will run on...

# Owner of the channel (used in some answers)
set goku_channel_owner(#channel1) "channel_owner"
set goku_channel_owner(#channel2) "channel_owner"
set goku_channel_owner(#channel3) "channel_owner"

# Should !saggio run on the channel? (default setting, 1 yes - 0 no)
set goku_channel_saggio(#channel1}  1
set goku_channel_saggio(#channel2)  1
set goku_channel_saggio(#channel3)  1

# Flood settings (not yet used...)
set goku_flood "5:50"

#
# End of preferences. As always don't edit below unless you know
# what you are doing (well, I don't, but I'm the programmer ;-)
#

# Header
set goku_header "\[gokuemu\]"

# help text for users
set goku_help "No help yet"

# help text for admins
set goku_admin_help "No help yet"

# Flag display
set switches {"off" "on"}

# Filenames
set goku_saggio_lib  "scripts/gokuemu_saggio.tcl"
set goku_fortune_lib "scripts/gokuemu_fortune.tcl"

# Binds
bind pub -|- !saggio        goku_saggio
bind pub -|- !saggio:       goku_saggio
bind pub n|n !setgoku       goku_setup
#bind pub -|- !fortune       goku_fortune

#
# End of settings, here starts the true code!
#

# First, some init code...

foreach tchan [array names goku_channel_saggio] {
  set goku_channel_saggio_lastmsg($tchan) ""
}

set goku_saggio_msgs {
  "Owner of this bot didn't installed correcly the data for Saggio!"
}
set goku_saggio_msgsl [llength $goku_saggio_msgs]

proc goku_library_load { library } {
  global goku_saggio_msgs goku_saggio_msgsl

  if {[info exists library]} {
    if {[file isfile $library]} {
      source $library
      putlog "$library loaded."
    } else {
      putlog "$library NOT loaded."
    }
  } else {
    putlog "$library NOT found!"
  }
}

proc goku_setup {nick uhost hand channel arg} {
  global botnick switches goku_header goku_channel_saggio goku_admin_help

  set channel [string tolower $channel]

  if {[info exists goku_channel_saggio($channel)] == 0} {
    return 0
  }

  if {[validchan $channel] && [onchan $botnick $channel]} {
    switch [string tolower [lindex $arg 0]] {
      "help" {
        putserv "PRIVMSG $channel :$goku_header $goku_admin_help"
      }

      "saggio" {
        putserv "PRIVMSG $channel :$goku_header Trigger: [lindex $switches [goku_setsaggio $channel]] (su $channel)"
      }
    }
  }
}

proc goku_setsaggio {channel} {
  global goku_channel_saggio

  if {$goku_channel_saggio($channel) == 1} {
    set goku_channel_saggio($channel) 0
  } else {
    set goku_channel_saggio($channel) 1
  }
  return $goku_channel_saggio($channel)
}

proc goku_saggio {nick uhost hand channel arg} {
  global botnick goku_header
  global goku_saggio_msgs goku_saggio_msgsl
  global goku_channel_saggio goku_channel_saggio_lastmsg

  set channel [string tolower $channel]

  if {[info exists goku_channel_saggio($channel)] == 1 && $goku_channel_saggio($channel) != 0} {
    if {$arg != "" && $arg != $goku_channel_saggio_lastmsg($channel) && [validchan $channel] && [onchan $botnick $channel] && [matchattr $hand |B $channel] == 0} {
      set goku_channel_saggio_lastmsg($channel) $arg

      putserv "PRIVMSG $channel :$nick, Il Capo dei Saggi dice: \"[goku_get_rnd_msg $nick $channel]\" "
    }
  }
}

proc goku_get_rnd_msg {nick channel} {
  global goku_channel_owner
  global goku_saggio_msgs goku_saggio_msgsl

  set msg [lindex $goku_saggio_msgs [rand $goku_saggio_msgsl]]
  regsub -all -nocase -- "%you" $msg "$nick" msg
  regsub -all -nocase -- "%owner" $msg $goku_channel_owner($channel) msg
  regsub -all -nocase -- "%someone" $msg [lindex [chanlist $channel] [rand [llength [chanlist $channel]]]] msg
  regsub -all -nocase -- "%number" $msg [rand 60] msg
  return $msg
}

goku_library_load $goku_saggio_lib

putlog "Loaded gokuemu.tcl v0.5 by gabriele@favrin.net"

